<?php

const DVG_APPOINTMENTS_CANCEL_APPOINTMENT_INPUT_STEP = 0;
const DVG_APPOINTMENTS_CANCEL_APPOINTMENT_CHECK_STEP = 1;
const DVG_APPOINTMENTS_CANCEL_APPOINTMENT_FINISHED_STEP = 2;

/**
 * Menu callback for cancel appointment form.
 */
function dvg_appointments_cancel_appointment() {
  /** @var \DvgAppointment $appointment */
  list($access, $appointment) = _dvg_appointments_cancel_appointment_parse_access_appointment();
  // We only need to check the access if there is an appointment found, if not
  // an extra step is rendered in which the user can enter an appointment number
  // and e-mail address.
  if ($appointment && !$access) {
    drupal_not_found();
    return '';
  }
  return drupal_get_form('dvg_appointments_cancel_appointment_form', $appointment);
}

/**
 * @return array
 *   The access boolean and appointment if present.
 */
function _dvg_appointments_cancel_appointment_parse_access_appointment() {
  $params = drupal_get_query_parameters();
  $access = $appointment = FALSE;
  if (isset($params['id'], $params['token']) && is_numeric($params['id']) && !empty($params['token'])) {
    $appointment = dvg_appointments_load_appointment($params['id']);
    if ($appointment) {
      $access_token = dvg_appointments_access_appointment_token($appointment);
      if ($access_token === $params['token'] && $appointment->getStart()->getTimestamp() > time()) {
        $access = TRUE;
      }
    }
  }
  return array($access, $appointment);
}

/**
 * Cancel appointment form.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 *
 * @return array
 *   Renderable array of the form.
 */
function dvg_appointments_cancel_appointment_form($form, &$form_state) {

  // Check if there is an appointment object in the build info.
  $appointment = $form_state['build_info']['args'][0];
  // Check the submitted values for any appointment information.
  if (!$appointment && isset($form_state['values']['appointment_number']['appointment'])) {
    $appointment = unserialize($form_state['values']['appointment_number']['appointment'], array('DvgAppointment'));
  }

  if (!isset($form_state['storage']['step'])) {
    $form_state['storage']['step'] = 0;
    // Skip the input step when using a direct cancel link.
    if ($appointment !== FALSE) {
      $form_state['storage']['step'] = DVG_APPOINTMENTS_CANCEL_APPOINTMENT_CHECK_STEP;
    }
  }

  $step = $form_state['storage']['step'];
  // Build the progressbar.
  $form['progressbar'] = _dvg_appointments_cancel_appointment_get_progressbar($step);

  $form['#attributes']['class'][] = 'appointment-webform';
  $next_step = t('Next page');
  switch ($step) {
    case DVG_APPOINTMENTS_CANCEL_APPOINTMENT_INPUT_STEP:
      $next_step = t('Check');
      _dvg_appointments_cancel_appointment_input_step($form, $form_state);
      break;

    case DVG_APPOINTMENTS_CANCEL_APPOINTMENT_CHECK_STEP:
      $next_step = t('Cancel appointment');
      _dvg_appointments_cancel_appointment_check_step($form, $form_state, $appointment);
      break;

    case DVG_APPOINTMENTS_CANCEL_APPOINTMENT_FINISHED_STEP:
      // Before reaching here the user is redirected to the completed page.
      break;

    default:
      watchdog('dvg_appointments', 'Got unexpected step in appointment cancel form: @step', array('@step' => $step), WATCHDOG_ERROR);
      throw new InvalidArgumentException('Unexpected step encountered, see the Drupal logs.');
  }

  _dvg_appointments_cancel_appointment_form_add_actions($form, $step, $next_step);
  return $form;
}

/**
 * Builds a progressbar for the appointment cancel form.
 *
 * @param int $step
 *   The current step of the form.
 * @return array
 *   Renderable array for the form progress bar.
 *
 * @throws \Exception
 */
function _dvg_appointments_cancel_appointment_get_progressbar($step) {
  // Recycle create form for progressbar settings.
  $node = node_load(functional_content_nid('dvg_appointments_create'));
  // Page nums are not 0 based.
  $page_num = $step + 1;
  $page_labels = array();
  $page_labels[DVG_APPOINTMENTS_CANCEL_APPOINTMENT_INPUT_STEP] = t('Start');
  $page_labels[DVG_APPOINTMENTS_CANCEL_APPOINTMENT_CHECK_STEP] = $node->webform['preview_title'] ? t($node->webform['preview_title']) : t('Preview');
  $page_labels[DVG_APPOINTMENTS_CANCEL_APPOINTMENT_FINISHED_STEP] = t($node->webform['progressbar_label_confirmation']);

  $page_count = count($page_labels);

  return array(
    '#theme' => 'webform_progressbar',
    '#node' => $node,
    '#page_num' => $page_num,
    '#page_count' => $page_count,
    '#page_labels' => $page_labels,
  );
}

/**
 * Adds the next/previous buttons to the change form.
 *
 * @param array $form
 *   Drupal form.
 * @param int $step
 *   Step number.
 * @param string $next_step
 *   Value of the "next" button.
 */
function _dvg_appointments_cancel_appointment_form_add_actions(&$form, $step, $next_step) {
  if ($step !== DVG_APPOINTMENTS_CANCEL_APPOINTMENT_FINISHED_STEP) {
    $form['#submit'] = array('dvg_appointments_cancel_appointment_form_submit');
    $form['actions'] = array(
      '#type' => 'actions',
      '#weight' => 1000,
    );
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => $next_step,
      '#attributes' => array(
        'class' => array(
          'webform-next',
          'button-primary',
        ),
      ),
    );
    if ($step > DVG_APPOINTMENTS_CANCEL_APPOINTMENT_CHECK_STEP) {
      $form['actions']['previous'] = array(
        '#type' => 'submit',
        '#value' => t('Previous'),
        '#attributes' => array(
          'class' => array(
            'webform-previous',
          ),
        ),
      );
    }
  }
}

/**
 * Cancel appointment form submit handler.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 */
function dvg_appointments_cancel_appointment_form_submit(&$form, &$form_state) {
  $step = $form_state['storage']['step'];
  if ($form_state['triggering_element']['#value'] === t('Previous')) {
    $step--;
  }
  else {
    $step++;
  }
  $form_state['storage']['step'] = $step;

  if ($step !== DVG_APPOINTMENTS_CANCEL_APPOINTMENT_FINISHED_STEP) {
    $form_state['rebuild'] = TRUE;
  }

  // Store all submitted data in storage.
  foreach ($form_state['values'] as $key => $value) {
    if (!in_array($key, array('submit', 'previous', 'form_build_id', 'form_token', 'form_id', 'op'), TRUE)) {
      $form_state['storage'][$key] = $value;
    }
  }

  // In the final step, delete the appointment submission and redirect to the
  // confirmation page.
  if ($step === DVG_APPOINTMENTS_CANCEL_APPOINTMENT_FINISHED_STEP && isset($form_state['values']['appointment'])) {
    $appointment = $form_state['values']['appointment'];
    // Delete the submission tied to the appointment. This will also delete
    // the local appointment.
    if ($appointment) {
      // We want to use webform submissions functions but we are currently not
      // on a webform.
      module_load_include('inc', 'webform', 'includes/webform.submissions');

      $node = node_load($appointment->getNodeId());
      $submission = webform_get_submission($appointment->getNodeId(), $appointment->getSubmissionId());
      if ($submission) {
        webform_submission_delete($node, $submission);
      }

      // Redirect to the completed page.
      drupal_goto(_dvg_appointments_get_cancel_form_url() . '/done');
    }
  }
}

/**
 * Create the form step where the user can enter the appointment details.
 *
 * @param array $form
 *   Drupal form.
 * @param $form_state
 *   The current state of the form.
 */
function _dvg_appointments_cancel_appointment_input_step(&$form, &$form_state) {
  $form['appointment_number'] = array(
    '#type' => 'wa_appointment',
    '#title' => t('Appointment'),
  );
  $form['e_mail'] = array(
    '#type' => 'wa_email',
    '#title' => t('E-mail'),
  );
}

/**
 * Overview page of the current appointment.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 * @param \DvgAppointment $appointment
 *   The appointment that is being changed.
 */
function _dvg_appointments_cancel_appointment_check_step(&$form, &$form_state, DvgAppointment $appointment) {
  $form['appointment'] = array(
    '#markup' => _dvg_appointments_table($appointment)
  );
  // This validator cancels the appointment in the client when this step is
  // submitted.
  $form['#validate'][] = 'dvg_appointments_form_cancel_appointment';
}

/**
 * Menu callback for the cancel appointment done step.
 *
 * @return array
 *   Renderable page.
 *
 * @throws \Exception
 */
function dvg_appointments_cancel_appointment_finished_step() {
  // Fake the progress bar to act "completed".
  $page['progressbar'] = _dvg_appointments_cancel_appointment_get_progressbar(DVG_APPOINTMENTS_CANCEL_APPOINTMENT_FINISHED_STEP);
  $completed_args = [];

  // Show a confirmation message to the user.
  drupal_set_message(t('Your appointment has been cancelled successfully.'));

  $create_nid = functional_content_nid('dvg_appointments_create');
  $cancel_link = l(t('Create appointment'), 'node/' . $create_nid);
  $completed_args['!create_link'] = $cancel_link;

  $completed_message = t('Use the following link to create a new appointment:<br>!create_link', $completed_args);
  $page['appointment_action_urls'] = [
    '#markup' => '<p class="appointment-action-urls">' . $completed_message . '</p>',
  ];

  return $page;
}

/**
 * Validation handler which cancels the appointment on the appointments client.
 *
 * @param array $form
 *   The form render array.
 * @param array $form_state
 *   The form state.
 */
function dvg_appointments_form_cancel_appointment(&$form, &$form_state) {
  // Try to load the appointment from the url parameters.
  [$access, $appointment] = _dvg_appointments_cancel_appointment_parse_access_appointment();
  // If no appointment found, check if the form state contains appointment data.
  if ($appointment === FALSE && isset($form_state['storage']['appointment_number']['appointment'])) {
    $appointment = unserialize($form_state['storage']['appointment_number']['appointment'], array('DvgAppointment'));
  }

  $client_api = dvg_appointments_get_client_api();
  if ($appointment === FALSE || !$client_api->delete_appointment($appointment)) {
    form_set_error('appointment', t('Unable to cancel the appointment, please try again later.'));
  }
  // Add the appointment data to the form_state values so it can be used in the
  // submit handler.
  $form_state['values']['appointment'] = $appointment;
}
